package net.asteasolutions.aos;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONObject;

import android.content.Context;
import android.graphics.drawable.Drawable;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.widget.Toast;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.ItemizedOverlay;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapView;
import com.google.android.maps.OverlayItem;

public class MainActivity extends MapActivity implements LocationListener {
    
   MyItemizedOverlay overlay;
   MapView mapView;
   LocationManager locationManager;
    
   @Override
   public void onCreate(Bundle savedInstanceState) {
       super.onCreate(savedInstanceState);
       setContentView(R.layout.main);

       // Getting the LocationManager instance.
       this.locationManager = (LocationManager) getSystemService(LOCATION_SERVICE);

       // Retrieve the MapView instance.
       this.mapView = (MapView) findViewById(R.id.map);

       // Enable the "+/-" buttons.
       this.mapView.setBuiltInZoomControls(true);

       // Create overlay for the pins.
       this.overlay = new MyItemizedOverlay(
	       getResources().getDrawable(R.drawable.current_location), getApplicationContext());
       
       // Get the last known device location.
       Location lastKnown = this.locationManager.getLastKnownLocation("gps");
       
       if (lastKnown != null) {
	   // Converting "double"s to "int"s.
	   GeoPoint initialLocation = new GeoPoint(
		   (int) (lastKnown.getLatitude() * 1e6), 
		   (int) (lastKnown.getLongitude() * 1e6));
	   
	   OverlayItem initialItem = new OverlayItem(initialLocation, "My item", ".");        
	   this.overlay.setItems(Arrays.asList(initialItem));
       } else {
	   // If no initial location is available, just set empty overlay list.
	   this.overlay.setItems(Collections.<OverlayItem>emptyList());
       }
       
       // Adding the overlay to the MapView.
       this.mapView.getOverlays().add(overlay);

       
       // Registering for updates whenever the GPS detects a location change.
       // XXX Values of "0" are not recommended here.. Just for testing purposes!
       this.locationManager.requestLocationUpdates("gps", 0, 0, this);
   }
   
   @Override
    protected void onDestroy() {
	super.onDestroy();
	
	// Unregistering for updates from LocationManager.
	this.locationManager.removeUpdates(this);
    }

    @Override
    protected boolean isRouteDisplayed() {
	return true;
    }
    
    @Override
    protected boolean isLocationDisplayed() {
        return true;
    }
    
    class MyItemizedOverlay extends ItemizedOverlay<OverlayItem> {
	
	private static final String GEOCODE_URL = "http://maps.googleapis.com/maps/api/geocode/json?latlng=%s,%s&sensor=true";
	ArrayList<OverlayItem> items = new ArrayList<OverlayItem>();
	Context context;
	
	MyItemizedOverlay(Drawable arg0, Context context) {
	    super(boundCenterBottom(arg0));
	    this.context = context;
	}
	
	void setItems(List<OverlayItem> items) {
	    this.items.clear();
	    this.items.addAll(items);
	    populate();
	}
	
	@Override
	public int size() {
	    return this.items.size();
	}
	
	@Override
	protected OverlayItem createItem(int i) {
	    return this.items.get(i);
	}
	
	@Override
	protected boolean onTap(int index) {
	    
	    // When an overlay item is clicked:
	    // 1) Get the item.
	    // 2) Get a Geocoder instance.
	    // 3) Try to reverse-geocode the current location in background.
	    // 4) If successful, display the first line of the address in a Toast.

	    
	    OverlayItem tapped = this.items.get(index);
	    
	    final GeoPoint tappedLocation = tapped.getPoint();
	    
	    new AsyncTask<Void, Void, String>() {

		@Override
		protected String doInBackground(Void... params) {
		    try {
			
			JSONObject response = parseGeocodeResponse(
				String.format(
					GEOCODE_URL, 
					(double) tappedLocation.getLatitudeE6() / 1e6, 
					(double) tappedLocation.getLongitudeE6() / 1e6));
			response = response.getJSONArray("results").getJSONObject(0);
			
			return response.getString("formatted_address");
		    } catch (Exception e) {
			Log.i(getPackageName(), "", e);
		    }
		    
		    return null;
		}
		
		protected void onPostExecute(String result) {
		    if (result != null) {
			Toast.makeText(getApplicationContext(), 
				"You are at: " + result, Toast.LENGTH_LONG).show();
		    }
		}
		
	    }.execute();
	    
	    return super.onTap(index);
	}
	
	private JSONObject parseGeocodeResponse(String url) throws Exception {

	    HttpClient httpclient = new DefaultHttpClient();
	    HttpGet httpget = new HttpGet(url);
	    HttpResponse response = httpclient.execute(httpget);
	    JSONObject json = new JSONObject();
	    HttpEntity entity = response.getEntity();
	    if (entity != null) {
		// A Simple JSON Response Read
		InputStream instream = entity.getContent();
		BufferedReader reader = new BufferedReader(new InputStreamReader(instream));
		StringBuilder sb = new StringBuilder();
		String line = null;
		try {
		    while ((line = reader.readLine()) != null) {
			sb.append(line + "\n");
		    }
		} finally {
		    instream.close();
		}
		String result = sb.toString();
		json = new JSONObject(result);
		instream.close();
	    }
	    return json;
	}
    }
    
    @Override
    public void onLocationChanged(Location location) {
	
	// Whenever the GPS reports a new device location:
	// 1) Create a new overlay item
	// 2) Remove all current pins.
	// 3) Add a new pin with the current device location.
	// 4) Refresh the map view.
	
	double latitude = location.getLatitude();
	double longitude = location.getLongitude();
	
	GeoPoint point = new GeoPoint((int)(latitude * 1e6), (int) (longitude * 1e6));
	
	OverlayItem item = new OverlayItem(point, "My item", ".");
	this.overlay.setItems(Arrays.asList(item));
	
	this.mapView.invalidate();
    }

    @Override
    public void onProviderDisabled(String provider) {
	// Nothing.
    }

    @Override
    public void onProviderEnabled(String provider) {
	// Nothing.
    }

    @Override
    public void onStatusChanged(String provider, int status, Bundle extras) {
	// Nothing.
    }
}